@@ -23,5 +23,12 @@
      bug-gnu-chess@gnu.org
      cracraft@ai.mit.edu, cracraft@stanfordalumni.org, cracraft@earthlink.net
 */
-#define BOOKSRC "book.pgn"
-#define BOOKBIN	"book.dat"
+#define BOOKSRC "book.src"
+#define BOOKBIN "book.bin"
+#define BOOKRUN "book.dat"
+#define BOOKRUNT "book.tmp"
+#define R_WHITE_WINS 1
+#define R_BLACK_WINS 2
+#define R_DRAW 3
+#define R_NORESULT 4
+#define BOOKDEPTH 20  /* We gobble this many moves from each PGN book game */
