@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <unistd.h>
+#include "version.h"
 #include "common.h"
 #include "eval.h"
 #ifdef UNIVERSAL
@@ -39,10 +40,10 @@ char fromboard[10];
 #define prompt ':'
 
 extern short stage, InChkDummy, terminal;
-static char inputstr[80], cmd[80], s[80], logfile[80], 
-            gamefile[80],resultstr[80];
+static char inputstr[128], cmd[128], file[128], s[128], logfile[128], 
+            gamefile[128],resultstr[128];
 
-char userinput[80],subcmd[80],setting[80],subsetting[80];
+char userinput[128],subcmd[128],setting[128],subsetting[128];
 
 
 #ifdef UNIVERSAL
@@ -73,6 +74,7 @@ void InputCmd ()
    int i;
    leaf *ptr; 
    int ncmds;
+   char *x;
 #ifdef UNIVERSAL
    char *p,c;
 #endif
@@ -80,8 +82,9 @@ void InputCmd ()
    CLEAR (flags, THINK);
 /*   if (!(flags&SELF) || ((flags & SELF) && (flags & ENDED))) { */
 #ifdef UNIVERSAL
-  strcpy(userinput,'\000');
-  strcpy(cmd,'\000');
+  memset(userinput,0,sizeof(userinput));
+  memset(cmd,0,sizeof(cmd));
+  memset(inputstr,0,sizeof(inputstr));
   if (1) {
     if (flags & UNIV) {
       userinput[0] = '\000';
@@ -110,17 +113,23 @@ void InputCmd ()
 	  printf ("%s (%d) %c ", color[board.side], (GameCnt+1)/2 + 1, prompt);
 	  fflush(stdout);
         }
+/* 
 	fgets (inputstr, 64, stdin);
-/*NEW */
 	inputstr[strlen(inputstr)-1] = '\000';
-/*END NEW */
+*/
+	if (fgets (inputstr, 64, stdin) && inputstr[0])
+	    inputstr[strlen(inputstr)-1] = '\000';
+	else
+	    SET (flags, QUIT);
 	cmd[0] = '\n';
 	strcpy(userinput,inputstr);
 	sscanf (inputstr, "%s %[^\n]", cmd, inputstr);
 	if (cmd[0] == '\n')
 	  return;
 #endif
-        ncmds = sscanf (userinput,"%s %s %s %s",cmd,subcmd,setting,subsetting);
+	cmd[0] = subcmd[0] = setting[0] = subsetting[0] = '\0';
+        ncmds = sscanf (userinput,"%s %s %s %[^\n]",
+			cmd,subcmd,setting,subsetting);
 #ifdef UNIVERSAL
         userinput[0] = '\000';
         printf ("%s (%d) %c ", color[board.side], (GameCnt+1)/2 + 1, prompt);
@@ -130,10 +139,12 @@ void InputCmd ()
       printf ("%s (%d) %c ", color[board.side], (GameCnt+1)/2 + 1, prompt);
       fflush(stdout);
       gets(userinput);
+      cmd[0] = subcmd[0] = setting[0] = subsetting[0] = '\0';
       ncmds = sscanf (userinput,"%s %s %s %s",cmd,subcmd,setting,subsetting);
     }
   }
 #endif
+
 /*
   printf("cmd=%s,subcmd=%s,setting=%s,subsetting=%s\n",cmd,subcmd,setting,subsetting);
   fflush(stdout);
@@ -143,22 +154,25 @@ void InputCmd ()
   printf("inputstr=%s\n",inputstr);
    printf("cmd = %s, inputstr = %s, subcmd = %s\n",cmd,inputstr,subcmd);
 */
+
    if (strcmp (cmd, "quit") == 0 || strcmp (cmd, "exit") == 0)
       SET (flags, QUIT);
    else if (strcmp (cmd, "help") == 0)
       ShowHelp ();
    else if (strcmp (cmd, "show") == 0)
       ShowCmd (inputstr);
-   else if (strcmp (cmd, "book") == 0) {
-      sscanf (inputstr, "%s %[^\n]", cmd, inputstr);
-printf("inputstr = %s\n",inputstr);
-      if (strncmp (inputstr, "compile", 7) == 0) {
-printf("calling genbook.\n");
-        genbook ();
-      } else if (strcmp (inputstr, "on") == 0) {
+   else if (strncmp (cmd, "book", 4) == 0) {
+      if (strncmp(inputstr, "add",3) == 0) {
+        sscanf (inputstr, "add %s", file);
+        if (access(file,F_OK) < 0) {
+	  printf("The syntax to add a new book is:\n\n\tbook add file.pgn\n");
+        } else {
+          BookPGNReadFromFile (file);
+	}
+      } else if (strncmp (inputstr, "on", 2) == 0) {
 	bookmode = BOOKBEST;
 	printf("book now on.\n");
-      } else if (strcmp (inputstr, "off") == 0) {
+      } else if (strncmp (inputstr, "off", 3) == 0) {
 	bookmode = BOOKOFF;
 	printf("book now off.\n");
       }
@@ -167,7 +181,7 @@ printf("calling genbook.\n");
    else if (strcmp (cmd, "version") == 0)
       ShowVersion ();
    else if (strcmp (cmd, "pgnsave") == 0)
-      PGNSaveToFile (inputstr);
+      PGNSaveToFile (inputstr,"");
    else if (strcmp (cmd, "pgnload") == 0)
       PGNReadFromFile (inputstr);
    else if (strcmp (cmd, "manual") == 0)
@@ -202,12 +216,32 @@ printf("calling genbook.\n");
 	;
    else if (strcmp (cmd, "easy") == 0)
 	;
+   else if (strcmp (cmd, "list") == 0) {
+	if (inputstr[0] == '?')
+	{
+	  printf("name    - list known players alphabetically\n");
+	  printf("score   - list by GNU best result first \n");
+	  printf("reverse - list by GNU worst result first\n");
+	} else {
+          sscanf (inputstr, "%s %[^\n]", cmd, inputstr);
+          if (inputstr == '\000') DBListPlayer("rscore");
+	  else DBListPlayer(inputstr);
+	}
+   }
    else if (strcmp (cmd, "post") == 0)
 	SET (flags, POST);
    else if (strcmp (cmd, "nopost") == 0)
 	CLEAR (flags, POST);
    else if (strcmp (cmd, "name") == 0) {
       strcpy(name, inputstr);
+      x = name;
+      while (*x != '\000') {
+        if (*x == ' ') {
+	  *x = '\000';
+	  break;
+	}
+        *x++;
+      }
       suffix = 0;
       for (;;) {
 	sprintf(logfile,"log.%03d",suffix);
@@ -225,22 +259,18 @@ printf("calling genbook.\n");
 	fclose(ofp); 
 	ofp = stdout;
         printf("Save to %s\n",gamefile);
-        PGNSaveToFile (gamefile);
+        PGNSaveToFile (gamefile, inputstr);
+	DBUpdatePlayer (name, inputstr);
      }
-#ifdef NEVER
-     if (flags & XBOARD) {
-	printf("tellics shout GNU 5.0 (Gazebo) ready to play!\n");
-	printf("tellics seek 5 1 rated f\n");
-	printf("tellics seek 3 0 rated f\n");
-	printf("tellics seek 1 0 rated f\n");
-	printf("tellics seek 3 1 rated f\n");
-	printf("tellics seek 1 1 rated f\n");
-     }
-#endif
    }	
    else if (strcmp (cmd, "rating") == 0) {
       sscanf(inputstr,"%d %d",&myrating,&opprating); 
       fprintf(ofp,"my rating = %d, opponent rating = %d\n",myrating,opprating); 
+      /* Change randomness of book based on opponent rating. */
+      /* Basically we play narrower book the higher the opponent */
+      if (opprating >= 1700) bookfirstlast = 2;
+      else if (opprating >= 1700) bookfirstlast = 2;
+      else bookfirstlast = 2;
    }
    else if (strcmp (cmd, "activate") == 0) {
 	CLEAR (flags, TIMEOUT);
@@ -251,6 +281,8 @@ printf("calling genbook.\n");
      NewPosition ();
      CLEAR (flags, MANUAL);
      CLEAR (flags, THINK);
+     bookloaded = 0;
+     myrating = opprating = 0;
    }
    else if (strcmp (cmd, "time") == 0) {
      sscanf (inputstr, "%s %[^\n]", s, inputstr);
@@ -308,6 +340,16 @@ printf("calling genbook.\n");
 	 SET (flags, XBOARD);	    /* like in xboard/winboard usage */
       }
    }
+   else if (strcmp (cmd, "protover") == 0)
+   {
+      if (flags & XBOARD) {
+	/* Note: change this if "analyze" or "draw" commands are added, etc. */
+	printf("feature setboard=1 analyze=0 ping=1 draw=0"
+	       " variants=\"normal\" myname=\"%s %s\" done=1\n",
+	       PROGRAM, VERSION);
+	fflush(stdout);
+      }
+   }
 #ifdef UNIVERSAL
   else if (strcmp(cmd,"universal") == 0) {
     if (strcmp(subcmd,"on") == 0) {
@@ -334,7 +376,8 @@ printf("calling genbook.\n");
       SearchDepth = 0;
       sscanf (inputstr, "%hd %f %hd", &TCMove, &TCTime, &TCinc);
       if (TCMove == 0) {
-	TCMove = 60;
+	TCMove =  35 /* MIN((5*(GameCnt+1)/2)+1,60) */;
+	printf("TCMove = %d\n",TCMove);
 	suddendeath = 1;
       } else
 	suddendeath = 0;
@@ -369,8 +412,25 @@ printf("calling genbook.\n");
    else if (strcmp (cmd, "epd") == 0)
    {
       ParseEPD (inputstr);
+      NewPosition();
+      ShowBoard();
       printf ("\n%s : Best move = %s\n", id, solution); 
    }
+   else if (strcmp (cmd, "setboard") == 0)
+   {
+      /* setboard uses FEN, not EPD, but ParseEPD will accept FEN too */
+      ParseEPD (inputstr);
+      NewPosition();
+   }
+   else if (strcmp (cmd, "ping") == 0)
+   {
+      /* If ping is received when we are on move, we are supposed to 
+	 reply only after moving.  In this version of GNU Chess, we
+	 never read commands while we are on move, so we don't have to
+	 worry about that here. */
+      printf("pong %s\n", inputstr);
+      fflush(stdout);
+   }
    else if (strcmp (cmd, "switch") == 0)
    {
       board.side = 1^board.side;
@@ -380,6 +440,8 @@ printf("calling genbook.\n");
    {
       SET (flags, THINK);
       CLEAR (flags, MANUAL);
+      CLEAR (flags, TIMEOUT);
+      CLEAR (flags, ENDED);
       computer = board.side;
 /*
       printf ("Computer is playing %s\n", computer ? "black" : "white");
@@ -397,7 +459,7 @@ printf("calling genbook.\n");
          if (!(flags & XBOARD))
            ShowBoard ();
        }
-       PGNSaveToFile ("game.log");
+       PGNSaveToFile ("game.log","");
     } else
        printf ("No moves to undo! \n");
    }
@@ -463,6 +525,11 @@ void ShowCmd (char *subcmd)
 */
    if (strcmp (cmd, "board") == 0)
       ShowBoard ();
+   else if (strcmp (cmd, "rating") == 0)
+   {
+      printf("My rating = %d\n",myrating);
+      printf("Opponent rating = %d\n",opprating);
+   } 
    else if (strcmp (cmd, "time") == 0)
       ShowTime ();
    else if (strcmp (cmd, "moves") == 0)
@@ -503,8 +570,18 @@ void ShowCmd (char *subcmd)
    {
       int s, wp, bp, wk, bk;
       short r, c, sq;
-      BitBoard *b;
+      BitBoard *b, x;
 
+#ifdef NEVER
+      x = DistMap[board.king[white]][3];
+      printf("Squares within 3 of white king:\n");
+      ShowBitBoard(&x);
+      getchar();
+      x = DistMap[board.king[black]][3];
+      printf("Squares within 3 of black king:\n");
+      ShowBitBoard(&x);
+      getchar();
+#endif
       phase = PHASE;
       GenAtaks ();
       FindPins (&pinned);
@@ -518,18 +595,19 @@ void ShowCmd (char *subcmd)
       bp = ScoreP (black);
       wk = ScoreK (white);
       bk = ScoreK (black);
-      printf ("White:  Mat:%4d/%4d  P:%d  N:%d  B:%d  R:%d  Q:%d  K:%d  Dev:%d  h:%d\n",
+      printf ("White:  Mat:%4d/%4d  P:%d  N:%d  B:%d  R:%d  Q:%d  K:%d  Dev:%d  h:%d x:%d\n",
 	board.pmaterial[white], board.material[white], wp, ScoreN(white), 
         ScoreB(white), ScoreR(white), ScoreQ(white), wk, 
-        ScoreDev(white), hunged[white]);
-      printf ("Black:  Mat:%4d/%4d  P:%d  N:%d  B:%d  R:%d  Q:%d  K:%d  Dev:%d  h:%d\n",
+        ScoreDev(white), hunged[white], ExchCnt[white]);
+      printf ("Black:  Mat:%4d/%4d  P:%d  N:%d  B:%d  R:%d  Q:%d  K:%d  Dev:%d  h:%d x:%d\n",
 	board.pmaterial[black], board.material[black], bp, ScoreN(black), 
         ScoreB(black), ScoreR(black), ScoreQ(black), bk,
-        ScoreDev(black), hunged[black]);
+        ScoreDev(black), hunged[black], ExchCnt[black]);
       printf ("Phase: %d\t", PHASE);
       s = ( EvaluateDraw () ? DRAWSCORE : Evaluate (-INFINITY, INFINITY));
       printf ("score = %d\n", s);
       printf ("\n");
+      return;
       for (r = 56; r >= 0; r -= 8)
       {
          printf ("     +---+---+---+---+---+---+---+---+\n");
@@ -544,6 +622,7 @@ void ShowCmd (char *subcmd)
          }
          printf ("\n");
       }
+
       printf ("     +---+---+---+---+---+---+---+---+\n");
       printf ("       a   b   c   d   e   f   g   h  \n");
    }
