@@ -50,9 +50,9 @@ BitBoard weaked[2];
 static short PawnSq[2][64] = 
 {
 {  0,  0,  0,  0,  0,  0,  0,  0,
-  -2, -2, -2, -5,-10, -2, -2, -2,
+   5,  5,  5,-10,-10,  5,  5,  5,
   -2, -2, -2,  6,  6, -2, -2, -2,
-   0,  0,  0, 15, 25,  0,  0,  0,
+   0,  0,  0, 25, 25,  0,  0,  0,
    2,  2, 12, 16, 16, 12,  2,  2,
    4,  8, 12, 16, 16, 12,  4,  4,
    4,  8, 12, 16, 16, 12,  4,  4,
@@ -61,16 +61,42 @@ static short PawnSq[2][64] =
    4,  8, 12, 16, 16, 12,  4,  4,
    4,  8, 12, 16, 16, 12,  4,  4,
    2,  2, 12, 16, 16, 12,  2,  2,
-   0,  0,  0, 15, 25,  0,  0,  0,
+   0,  0,  0, 25, 25,  0,  0,  0,
   -2, -2, -2,  6,  6, -2, -2, -2,
-  -2, -2, -2, -5,-10, -2, -2, -2,
+   5,  5,  5,-10,-10,  5,  5,  5,
    0,  0,  0,  0,  0,  0,  0,  0}
 };
 
 static const short Passed[2][8] =
 { { 0, 48, 48, 96, 144, 192, 240, 0}, {0, 240, 192, 144, 96, 48, 48, 0} };
-/*{ { 0, 40, 40, 60, 100, 160, 240, 0}, {0, 240, 160, 100, 60, 40, 40, 0} };*/
+/* Credit if passed pawn blocked by enemy piece one square ahead */
+static const short Passed0[2][8] =
+{ { 0, 12, 12, 24, 36, 48, 60, 0}, {0, 60, 48, 36, 24, 12, 12, 0} };
+/* Penalties for one or more isolated pawns on a given file */
+static const short isolani_normal[8] = {
+  -8, -10, -12, -14, -14, -12, -10, -8
+};
+/* Penalties if the file is half-open (i.e. no enemy pawns on it) */
+static const short isolani_weaker[8] = {
+  -22, -24, -26, -28, -28, -26, -24, -22
+};
 
+static const char outside_passed[128] ={ 96, 48, 48, 48, 45, 42, 40, 40,
+                                    36, 36, 32, 32, 28, 28, 24, 24,
+                                    20, 20, 16, 16, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12,
+                                    12, 12, 12, 12, 12, 12, 12, 12};
 static const short rank7[2] = { 6, 1 };
 static const short rank8[2] = { 7, 0 };
 static const BitBoard d2e2[2] =
@@ -108,6 +134,9 @@ inline int ScoreP (short side)
    short nfile[8];
    short EnemyKing;
    BitBoard c, t, p, blocker, *e;
+/*
+   BitBoard holes, d;
+*/
    PawnSlot *ptable;
 
    if (board.b[side][pawn] == NULLBITBOARD)
@@ -116,6 +145,22 @@ inline int ScoreP (short side)
    EnemyKing = board.king[xside];
    p = board.b[xside][pawn];
    c = t = board.b[side][pawn];
+/*
+   d = c;
+   if (side == white) {
+     d = d >> 8;
+     holes = (d & ( c >> 7) & ~FileBit[0]) | (d & ( c >> 9) & ~FileBit[7]);
+     holes = d & ~holes;
+   } else {
+     d = d << 8;
+     holes = (d & ( c << 7) & ~FileBit[7]) | (d & ( c << 9) & ~FileBit[0]);
+     holes = d & ~holes;
+   }
+   printf("Holes for %s for:\n",side==white?"white":"black");
+   ShowBoard();
+   ShowBitBoard(&holes);
+   getchar();
+*/
    ptable = PawnTab[side] + (PawnHashKey & PHashMask);
    TotalPawnHashCnt++;
    if (ptable->phase == phase && ptable->pkey == KEY(PawnHashKey))
@@ -144,7 +189,32 @@ inline int ScoreP (short side)
 	     (side == black && (FromToRay[sq][sq&7] & c) == 0))
 	 {
             passed[side] |= BitPosArray[sq];
-            s += (Passed[side][RANK(sq)] * phase) / 12;
+            if (side == white) {
+              if (board.friend[xside] & BitPosArray[sq+8])
+                  s += (Passed0[side][RANK(sq)] * phase) / 12;
+              else
+                  s += (Passed[side][RANK(sq)] * phase) / 12;
+            } else if (side == black) {
+              if (board.friend[xside] & BitPosArray[sq-8])
+                  s += (Passed0[side][RANK(sq)] * phase) / 12;
+              else
+                  s += (Passed[side][RANK(sq)] * phase) / 12;
+	    }
+            if (ENDING) {
+              /* Check for outside passed pawns and give additional bonus */
+                if ( ( (FILE(sq) == 0 || FILE(sq) == 1) &&
+                     ( (FileBit[FILE(sq)+1]&
+                        (board.b[side][pawn]|board.b[xside][pawn]))
+                       == NULLBITBOARD)) ||
+                     ( (FILE(sq) == 6 || FILE(sq) == 7) &&
+                     ( (FileBit[FILE(sq)+1]&
+                        (board.b[side][pawn]|board.b[xside][pawn])))))
+                   s += outside_passed[nbits(
+                         board.b[xside][knight]|
+                         board.b[xside][bishop]|
+                         board.b[xside][rook]|
+                         board.b[xside][queen])];
+            }
 	 }
       }
 
@@ -194,11 +264,39 @@ inline int ScoreP (short side)
       /* Isolated pawns */
       if (nfile[i] && (!(c & IsolaniMask[i])))
       {
-         s += ISOLATEDPAWN * nfile[i];
+	 /* Isolated on a half-open file */
+         if (!(FileBit[i] & board.b[xside][pawn]))
+	   s += isolani_weaker[i] * nfile[i];
+	 else /* Normal isolated pawn */
+           s += isolani_normal[i] * nfile[i];
 	 weaked[side] |= (c & FileBit[i]);
       }
    }
 
+
+  if (computerplays == side) {
+
+    /* Penalize having eight pawns */
+    if (nbits(board.b[computerplays][pawn]) == 8)
+        s += EIGHT_PAWNS;
+
+    /* Detect stonewall formation in enemy */
+    if (nbits(stonewall[xside] & board.b[xside][pawn]) == 3)
+      s += STONEWALL;
+
+    /* Locked pawns */
+    n = 0;
+    if (side == white)
+      n = nbits((c >> 8) & board.b[xside][pawn] &
+               (rings[0]|rings[1]|rings[2]));
+    else
+      n = nbits((c << 8) & board.b[xside][pawn] &
+               (rings[0]|rings[1]|rings[2]));
+    if (n > 1)
+      s += n * LOCKEDPAWNS;
+  }
+
+
    /* Save the score into the pawn hash table */ 
    ptable->pkey = KEY(PawnHashKey);
    ptable->passed = passed[side];
@@ -278,8 +376,8 @@ phase2:
       }
    }
 
-   /* If both sides are castled on different sides, bonus for pawn storms */
-/*
+  /* If both sides are castled on different sides, bonus for pawn storms */
+  c = board.b[side][pawn];
   if (abs (FILE (board.king[side]) - FILE (board.king[xside])) >= 4 &&
 	PHASE < 6)
   {
@@ -292,7 +390,6 @@ phase2:
         s += 10 * (5 - distance[sq][board.king[xside]]);
      }
   }
-*/
 
    return (s);
 }
@@ -331,9 +428,8 @@ static const short Outpost[2][64] =
 };
 
 
-inline int CTL(short , short , short , short , short , short , short );
-inline int CTL(short sq, short piece, short side, short EnemyKing,
-	short bonus_center, short bonus_attack, short bonus_mobility)
+inline int CTL(short , short , short);
+inline int CTL(short sq, short piece, short side)
 /***************************************************************************
  *
  *  Return a score corresponding to the number of squares in the bitboard
@@ -343,24 +439,31 @@ inline int CTL(short sq, short piece, short side, short EnemyKing,
  *
  ***************************************************************************/
 {
-  short s, n;
+  short s, n, EnemyKing, FriendlyKing;
   BitBoard controlled;
 
   s = 0;
 
-  controlled = AttackFrom (sq, piece, side);
+  EnemyKing = board.king[1^side];
+  FriendlyKing = board.king[side];
+
+  controlled = AttackXFrom (sq, side);
 
   /* Center control */
-  n = nbits (controlled & extended_center);
-  s += n * bonus_center;
+  n = nbits (controlled & (rings[0]|rings[1]));
+  s += n;
 
   /* Attacks against enemy king */
   n = nbits (controlled & DistMap[EnemyKing][2]);
-  s += n * bonus_attack;
+  s += n;
+
+  /* Defenses for friendly kin */
+  n = nbits (controlled & DistMap[FriendlyKing][2]);
+  s += n;
 
   /* Mobility */
   n = nbits(controlled);
-  s += n * bonus_mobility;
+  s += n;
 
   return (s);
 }
@@ -394,14 +497,14 @@ inline int ScoreN (short side)
       CLEARBIT (c, sq);
 
       /* Control */
-      s1 = CTL(sq,knight,side,EnemyKing,
-	KNIGHT_CENTER_BONUS,KNIGHT_ATTACK_BONUS,KNIGHT_MOBILITY_BONUS);
+      s1 = CTL(sq,knight,side);
   
-      if ( (BitPosArray[sq] & (FileBit[0]|FileBit[7])) != NULLBITBOARD)
+      if ( (BitPosArray[sq] & 
+	    (FileBit[0]|FileBit[7]|RankBit[0]|RankBit[7])) != NULLBITBOARD)
 	s1 += KNIGHTONRIM;
 
       if (Outpost[side][sq] && 
-	  !(t & IsolaniMask[FILE(sq)] & PassedPawnMask[side][sq]))
+	  !(t & IsolaniMask[FILE(sq)] & PassedPawnMask[side][sq]) )
       {
          s1 += OUTPOSTKNIGHT;
 
@@ -456,8 +559,7 @@ inline int ScoreB (short side)
       n++;
 
       /* Control */
-      s1 = CTL(sq,bishop,side,EnemyKing,
-	BISHOP_CENTER_BONUS,BISHOP_ATTACK_BONUS,BISHOP_MOBILITY_BONUS);
+      s1 = CTL(sq,bishop,side);
 
       /*  Outpost bishop */
       if (Outpost[side][sq] && 
@@ -497,11 +599,9 @@ inline int ScoreB (short side)
       s += s1;
    }
 
-#ifdef NEVER
    /* Doubled bishops */
    if (n > 1)            
       s += DOUBLEDBISHOPS;
-#endif
 
    return (s);
 
@@ -588,15 +688,15 @@ inline int ScoreR (short side)
       CLEARBIT (c, sq);
 
       /* Control */
-      if (Developed[side])
-        s1 = CTL(sq,rook,side,EnemyKing,
-	   ROOK_CENTER_BONUS,ROOK_ATTACK_BONUS,ROOK_MOBILITY_BONUS);
+      s1 = CTL(sq,rook,side);
 
       fyle = FILE(sq);
       if (PHASE < 7)
       {
          if (!(board.b[side][pawn] & FileBit[fyle]))
          {
+	    if (fyle == 5 && FILE(board.king[xside])>=E_FILE)
+	      s1 += ROOKLIBERATED;
             s1 += ROOKHALFFILE;
             if (!(board.b[xside][pawn] & FileBit[fyle]))
                s1 += ROOKOPENFILE;
@@ -693,15 +793,18 @@ inline int ScoreQ (short side)
    c = board.b[side][queen];
    EnemyKing = board.king[xside];
 
+   /* Try to keep our queen on the board for attacking purposes. */
+   if (side == computer)
+     if (c == NULLBITBOARD)
+        s1 += QUEEN_NOT_PRESENT;
+
    while (c)
    {
       sq = leadz (c);
       CLEARBIT (c, sq);
 
       /* Control */
-      if (Developed[side])
-        s1 = CTL(sq,queen,side,EnemyKing,
-	   QUEEN_CENTER_BONUS,QUEEN_ATTACK_BONUS,QUEEN_MOBILITY_BONUS);
+      s1 += CTL(sq,queen,side);
 
       if (distance[sq][EnemyKing] <= 2)
          s1 += QUEENNEARKING;
@@ -723,14 +826,14 @@ inline int ScoreQ (short side)
 
 static const short KingSq[64] =
 {
-   32, 32, 24, 16, 16, 24, 32, 32,
+   24, 24, 24, 16, 16,  0, 32, 32,
    24, 20, 16, 12, 12, 16, 20, 24,
    16, 12,  8,  4,  4,  8, 12, 16,
    12,  8,  4,  0,  0,  4,  8, 12,
    12,  8,  4,  0,  0,  4,  8, 12,
    16, 12,  8,  4,  4,  8, 12, 16,
    24, 20, 16, 12, 12, 16, 20, 24,
-   32, 32, 24, 16, 16, 24, 32, 32
+   24, 24, 24, 16, 16,  0, 32, 32
 };
 
 static const short EndingKing[64] =
@@ -745,7 +848,7 @@ static const short EndingKing[64] =
    0,  6, 12, 18, 18, 12,  6,  0
 };
 
-static short pawncover[9] = { -60, -30, 0, 5, 10, 15, 20, 25, 30 };
+static short pawncover[9] = { -60, -30, 0, 5, 30, 30, 30, 30, 30 };
 static const short factor[9] = { 7, 8, 8, 7, 6, 5, 4, 2, 0, };
 static const BitBoard Corner[2] = { 0xC3C3000000000000ULL,
 				    0x000000000000C3C3ULL };
@@ -758,26 +861,81 @@ inline int ScoreK (short side)
  *  3.  king on open file.
  *  4.  Uncastled king.
  *  5.  Open rook file via Ng5 or Bxh7 sac.
+ *  6.  No major or minor piece in the king's quadrant.
  *
  ***************************************************************************/
 {
-   short s, sq, sq1, n, xside, file, fsq;
-   BitBoard b;
+   short s, sq, sq1, n, n1, n2, xside, file, fsq, rank;
+   BitBoard b, x, x1, x2;
 
    s = 0;
    xside = 1^side;
    sq = board.king[side];
+   file = FILE (sq);
+   rank = RANK (sq);
    KingSafety[side] = 0;
    if (!ENDING)
    { 
 
-      /*s += KingSq[sq];*/
       s += ((6 - phase) * KingSq[sq] + phase * EndingKing[sq]) / 6;
 
-      n = nbits (MoveArray[king][sq] & board.b[side][pawn]);
+      /* After castling kingside, reward having all 3 pawns in front but not if
+	 there is a threatening pawn. This permits the freeing move F3/F6. */
+	 
+#ifdef NEVER
+      pawncover[3] = 5;
+      if (board.castled[side]) {
+	if ((side == black && ((BitPosArray[G5] & board.b[white][pawn])||
+			       (BitPosArray[E5] & board.b[white][pawn]))) ||
+	    (side == white && ((BitPosArray[G4] & board.b[black][pawn])||
+			       (BitPosArray[E4] & board.b[black][pawn])))) 
+	  ;
+	else
+	  pawncover[3] = 30; 
+      }
+#endif
+      if (side == white)
+        n = nbits (MoveArray[king][sq] & board.b[side][pawn] & RankBit[rank+1]);
+      else
+        n = nbits (MoveArray[king][sq] & board.b[side][pawn] & RankBit[rank-1]);
       s += pawncover[n];
 
-      file = FILE (sq);
+      /* Penalize compromised wing pawn formations prior to castle. */
+      if (!board.castled[side]) {
+        n = -1;
+        if (side == white) {
+          /* King on original square and unmoved */
+	  if (sq == 4 && Mvboard[sq] == 0) {
+	    /* Kingside - rook on original square and unmoved. */
+	    if ( (board.b[side][rook] & BitPosArray[H1])!=NULLBITBOARD &&
+		 Mvboard[H1] == 0)
+                   n = nbits (MoveArray[king][G1] & 
+			      board.b[side][pawn] & RankBit[rank+1]);
+	    /* Queenside */
+	    if ( (board.b[side][rook] & BitPosArray[A1])!=NULLBITBOARD &&
+		 Mvboard[A1] == 0)
+                   n = nbits (MoveArray[king][C1] & 
+			      board.b[side][pawn] & RankBit[rank+1]);
+          }
+   	} else {
+	  if (sq == 60 && Mvboard[sq] == 0) {
+	    /* Kingside */
+	    if ( (board.b[side][rook] & BitPosArray[H8])!=NULLBITBOARD &&
+		 Mvboard[H8] == 0)
+                   n = nbits (MoveArray[king][G8] & 
+			      board.b[side][pawn] & RankBit[rank-1]);
+	    /* Queenside */
+	    if ( (board.b[side][rook] & BitPosArray[A8])!=NULLBITBOARD &&
+		 Mvboard[A8] == 0)
+                   n = nbits (MoveArray[king][C8] & 
+			      board.b[side][pawn] & RankBit[rank-1]);
+          }
+	}
+        pawncover[3] = 30;
+	/* Penalize breaking the wing pawn formations prior to castle */
+	if (n != -1) s += pawncover[n];
+      }
+
       if (side == computer && file >= F_FILE && 
 		!(FileBit[G_FILE] & board.b[side][pawn]))
       {
@@ -787,8 +945,11 @@ inline int ScoreK (short side)
             s += GOPEN;
       }
 
+      /* No friendly pawns on this king file */
       if (!(FileBit[file] & board.b[side][pawn]))
          s += KINGOPENFILE;
+
+      /* No enemy pawns on this king file */
       if (!(FileBit[file] & board.b[xside][pawn]))
          s += KINGOPENFILE1;
 
@@ -841,9 +1002,49 @@ inline int ScoreK (short side)
 	  }
 	}
       }
-      if (side == computer && file >= G_FILE && board.b[xside][queen] &&
-	    !(board.b[side][pawn] & FileBit[7]))
+      if (side == computer) {
+
+	/* Stock piece sacrifice on h file */
+
+	if (file >= E_FILE && board.b[xside][queen] && board.b[xside][rook] &&
+	    !((board.b[side][pawn]|board.b[xside][pawn]) & FileBit[7]))
          s += HOPEN;
+	
+	/* King trapping rook */
+        if (side == white) {
+	  if (file > E_FILE) {
+	    if (board.b[side][rook]&mask_kr_trapped_w[H_FILE-file]) {
+		s += ROOKTRAPPED;
+/*
+		printf("Trapped %s rook at %s\n",side==white?"white":"black",algbr[sq]); ShowBoard(); getchar();
+*/
+	    }
+	  } else if (file < D_FILE) {
+	    if (board.b[side][rook]&mask_qr_trapped_w[file]) {
+		s += ROOKTRAPPED;
+/*
+		printf("Trapped %s rook at %s\n",side==white?"white":"black",algbr[sq]); ShowBoard(); getchar();
+*/
+	    }
+	  }
+	} else {
+	  if (file > E_FILE) {
+	    if (board.b[side][rook]&mask_kr_trapped_b[H_FILE-file]) {
+		s += ROOKTRAPPED;
+/*
+		printf("Trapped %s rook at %s\n",side==white?"white":"black",algbr[sq]); ShowBoard(); getchar();
+*/
+	    }
+	  } else if (file < D_FILE) {
+	    if (board.b[side][rook]&mask_qr_trapped_b[file]) {
+		s += ROOKTRAPPED;
+/*
+		printf("Trapped %s rook at %s\n",side==white?"white":"black",algbr[sq]); ShowBoard(); getchar();
+*/
+	    }
+	  }
+	}
+      }
 
       /* Penalize open lines to the king */
 /*
@@ -856,7 +1057,7 @@ inline int ScoreK (short side)
 */
 
       /* Don't give fianchetto target for advanced pawns */
-      if (file >= E_FILE) {
+      if (file > E_FILE && FILE(board.king[xside]) < D_FILE) {
          if (side == white) fsq = G3; else fsq = G6;
 	 if ((BitPosArray[fsq] & board.b[side][pawn]) != NULLBITBOARD)
             if (((BitPosArray[F4]|BitPosArray[H4]|
@@ -864,7 +1065,7 @@ inline int ScoreK (short side)
 	      & board.b[xside][pawn]) != NULLBITBOARD) 
 	        s += FIANCHETTO_TARGET;	
       }
-      if (file <= E_FILE) {
+      if (file < E_FILE && FILE(board.king[xside]) > E_FILE) {
          if (side == white) fsq = B3; else fsq = B6;
 	 if ((BitPosArray[fsq] & board.b[side][pawn]) != NULLBITBOARD)
             if (((BitPosArray[A4]|BitPosArray[C4]|
@@ -873,14 +1074,26 @@ inline int ScoreK (short side)
 	        s += FIANCHETTO_TARGET;	
       }
 
+      /* No major/minor piece in king's quadrant */
+      /* First identify the quadrant */
+      x = boardhalf[side] & boardside[file<=D_FILE];
+      /* Now identify the number of non-pawn enemy in quadrant */
+      n1 = nbits(x & (board.friend[xside]));
+      if (n1 > 0) {
+        /* Now identify the number of non-pawn friends in quadrant */
+        n2 = nbits(x & (board.friend[side] & ~board.b[side][pawn] & 
+	 	~board.b[side][king]));
+        if (n1 > n2)
+	  s += (n1 - n2) * KING_DEFENDER_DEFICIT;
+      }
+      
       KingSafety[side] = s;
       s = (s * factor[phase]) / 8;
    }
    else
    {
       s += EndingKing[sq];
-      s += CTL(sq,king,side,board.king[xside],
-	   KING_CENTER_BONUS,KING_ATTACK_BONUS,KING_MOBILITY_BONUS);
+      s += CTL(sq,king,side);
       b = (board.b[white][pawn] | board.b[black][pawn]);
       while (b)
       {
@@ -897,6 +1110,20 @@ inline int ScoreK (short side)
       /* Attack on weak opponent pawns */
       if (MoveArray[king][sq] & weaked[xside])
          s += ATAKWEAKPAWN * 2;
+
+   }
+
+    if (phase >= 4) {
+      /* Weak back rank */
+      if (side == white) {
+        if (sq < A2) 
+	  if (!(MoveArray[king][sq] & (~board.b[side][pawn] & RankBit[1]))) 
+	    s += KING_BACK_RANK_WEAK;
+      } else {
+	if (sq > H7) 
+	  if (!(MoveArray[king][sq] & (~board.b[side][pawn] & RankBit[6]))) 
+	    s += KING_BACK_RANK_WEAK;
+      }
    }
 
    pscore[sq] = s;
@@ -1138,26 +1365,25 @@ inline int ScoreDev (short side)
    /* Calculate whether we are developed */
    c = (board.b[side][knight] & nn[side]) | (board.b[side][bishop] & bb[side]);
    if (c == NULLBITBOARD) Developed[side] = true; else Developed[side] = false;
+   s += nbits(c) * -8;
 
-   /* If we are castled or the king is in the corner, just return 
-      otherwise we are not castled and so exact the penalty */
-   if (board.castled[side] || (board.b[side][king] & corner))
+   /* If we are castled or beyond the 10th move, no more ScoreDev */
+   if (board.castled[side] || GameCnt/2+1 >= 20)
       return (s);
-   else  
-     s += NOTCASTLED;
+
+   s += NOTCASTLED;
 
    /* If the king is moved, nail it, otherwise check rooks */
    if (Mvboard[board.king[side]] > 0) 
       s += KINGMOVED;
-   else {
-     /* Discourage rook moves */
-     c = board.b[side][rook];
-     while (c) {
-       sq = leadz(c);
-       CLEARBIT(c, sq);
-       if (Mvboard[sq] > 0)
-         s += ROOKMOVED;
-     }
+
+   /* Discourage rook moves */
+   c = board.b[side][rook];
+   while (c) {
+     sq = leadz(c);
+     CLEARBIT(c, sq);
+     if (Mvboard[sq] > 0)
+       s += ROOKMOVED;
    }
 
    /* Penalize a queen that moves at all */
@@ -1185,7 +1411,16 @@ inline int ScoreDev (short side)
      sq = leadz(c);
      CLEARBIT(c, sq);
      if (Mvboard[sq] > 0) 
-	s += Mvboard[sq] * EARLYWINGPAWNMOVE;
+	s += EARLYWINGPAWNMOVE;
+   }
+
+   /* Discourage any repeat center pawn moves */
+   c = board.b[side][pawn] & (FileBit[2]|FileBit[3]|FileBit[4]|FileBit[5]);
+   while (c) {
+     sq = leadz(c);
+     CLEARBIT(c, sq);
+     if (Mvboard[sq] > 1) 
+	s += EARLYCENTERPREPEAT;
    }
 
    return (s);
@@ -1314,7 +1549,9 @@ int Evaluate (int alpha, int beta)
    else
    {
       EvalCnt++;
+/*
       FindPins (&pinned);
+*/
       s1 = 0;
       for (piece = knight; piece < king; piece++)
       {
