@@ -53,6 +53,7 @@ void Initialize ()
    InitSquarePawnMask ();
    InitBitCount ();
    InitRotAtak ();
+   InitRandomMasks ();
 
    InitDistance ();
    InitVars ();
@@ -330,6 +331,22 @@ void InitRankFileBit ()
 }
 
 
+void InitRandomMasks ()
+{
+  mask_kr_trapped_w[0]=BitPosArray[H2];
+  mask_kr_trapped_w[1]=BitPosArray[H1]|BitPosArray[H2];
+  mask_kr_trapped_w[2]=BitPosArray[G1]|BitPosArray[H1]|BitPosArray[H2];
+  mask_qr_trapped_w[0]=BitPosArray[A2];
+  mask_qr_trapped_w[1]=BitPosArray[A1]|BitPosArray[A2];
+  mask_qr_trapped_w[2]=BitPosArray[A1]|BitPosArray[B1]|BitPosArray[A2];
+  mask_kr_trapped_b[0]=BitPosArray[H7];
+  mask_kr_trapped_b[1]=BitPosArray[H8]|BitPosArray[H7];
+  mask_kr_trapped_b[2]=BitPosArray[H8]|BitPosArray[G8]|BitPosArray[H7];
+  mask_qr_trapped_b[0]=BitPosArray[A7];
+  mask_qr_trapped_b[1]=BitPosArray[A8]|BitPosArray[A7];
+  mask_qr_trapped_b[2]=BitPosArray[A8]|BitPosArray[B8]|BitPosArray[A7];
+}
+
 void InitPassedPawnMask ()
 /**************************************************************************
  *
@@ -674,27 +691,75 @@ void InitVars ()
    SETBIT (stonewall[black], E6); /* SMC */
    SETBIT (stonewall[black], F5); /* SMC */
 
-   SETBIT (center, D4);
-   SETBIT (center, D5);
-   SETBIT (center, E4);
-   SETBIT (center, E5);
+   SETBIT (rings[0], D4);
+   SETBIT (rings[0], D5);
+   SETBIT (rings[0], E4);
+   SETBIT (rings[0], E5);
+   SETBIT (rings[1], C3);
+   SETBIT (rings[1], D3);
+   SETBIT (rings[1], E3);
+   SETBIT (rings[1], F3);
+   SETBIT (rings[1], C4);
+   SETBIT (rings[1], F4);
+   SETBIT (rings[1], C5);
+   SETBIT (rings[1], F5);
+   SETBIT (rings[1], C6);
+   SETBIT (rings[1], D6);
+   SETBIT (rings[1], E6);
+   SETBIT (rings[1], F6);
+   SETBIT (rings[2], B2);
+   SETBIT (rings[2], C2);
+   SETBIT (rings[2], D2);
+   SETBIT (rings[2], E2);
+   SETBIT (rings[2], F2);
+   SETBIT (rings[2], G2);
+   SETBIT (rings[2], B3);
+   SETBIT (rings[2], G3);
+   SETBIT (rings[2], B4);
+   SETBIT (rings[2], G4);
+   SETBIT (rings[2], B5);
+   SETBIT (rings[2], G5);
+   SETBIT (rings[2], B6);
+   SETBIT (rings[2], G6);
+   SETBIT (rings[2], B7);
+   SETBIT (rings[2], C7);
+   SETBIT (rings[2], D7);
+   SETBIT (rings[2], E7);
+   SETBIT (rings[2], F7);
+   SETBIT (rings[2], G7);
+   SETBIT (rings[3], A1);
+   SETBIT (rings[3], B1);
+   SETBIT (rings[3], C1);
+   SETBIT (rings[3], D1);
+   SETBIT (rings[3], E1);
+   SETBIT (rings[3], F1);
+   SETBIT (rings[3], G1);
+   SETBIT (rings[3], H1);
+   SETBIT (rings[3], A2);
+   SETBIT (rings[3], H2);
+   SETBIT (rings[3], A3);
+   SETBIT (rings[3], H3);
+   SETBIT (rings[3], A4);
+   SETBIT (rings[3], H4);
+   SETBIT (rings[3], A5);
+   SETBIT (rings[3], H5);
+   SETBIT (rings[3], A6);
+   SETBIT (rings[3], H6);
+   SETBIT (rings[3], A7);
+   SETBIT (rings[3], H7);
+   SETBIT (rings[3], A8);
+   SETBIT (rings[3], B8);
+   SETBIT (rings[3], C8);
+   SETBIT (rings[3], D8);
+   SETBIT (rings[3], E8);
+   SETBIT (rings[3], F8);
+   SETBIT (rings[3], G8);
+   SETBIT (rings[3], H8);
 
-   SETBIT (extended_center, C3);
-   SETBIT (extended_center, D3);
-   SETBIT (extended_center, E3);
-   SETBIT (extended_center, F3);
-   SETBIT (extended_center, C4);
-   SETBIT (extended_center, D4);
-   SETBIT (extended_center, E4);
-   SETBIT (extended_center, F4);
-   SETBIT (extended_center, C5);
-   SETBIT (extended_center, D5);
-   SETBIT (extended_center, E5);
-   SETBIT (extended_center, F5);
-   SETBIT (extended_center, C6);
-   SETBIT (extended_center, D6);
-   SETBIT (extended_center, E6);
-   SETBIT (extended_center, F6);
+   boardhalf[white] = RankBit[0]|RankBit[1]|RankBit[2]|RankBit[3];
+   boardhalf[black] = RankBit[4]|RankBit[5]|RankBit[6]|RankBit[7];
+   boardside[ks] = FileBit[4]|FileBit[5]|FileBit[6]|FileBit[7];
+   boardside[qs] = FileBit[0]|FileBit[1]|FileBit[2]|FileBit[3];
 
    board.flag |= (WCASTLE | BCASTLE);
    board.side = white;
@@ -850,4 +915,5 @@ void NewPosition ()
    TTClear ();
    PTClear ();
    nmovesfrombook = 0;
+   ExchCnt[white] = ExchCnt[black] = 0;
 }
