@@ -85,6 +85,7 @@ void MakeMove (short side, int *move)
 
    if (tpiece != 0)		/* Capture */
    {
+      ExchCnt[side]++;
       CLEARBIT (board.b[xside][tpiece], t);
       *move |= (tpiece << 15);
       HashKey ^= hashcode[xside][tpiece][t];
@@ -109,6 +110,7 @@ void MakeMove (short side, int *move)
 
    if (*move & ENPASSANT)	/* En passant */
    {
+      ExchCnt[side]++;
       epsq = board.ep + (side == white ? - 8 : 8);
       CLEARBIT (board.b[xside][pawn], epsq);
       CLEARBIT (board.blockerr90, r90[epsq]);
@@ -299,6 +301,7 @@ void UnmakeMove (short side, int *move)
    /* if capture, put back the captured piece */
    if (*move & CAPTURE)
    {
+      ExchCnt[side]--;
       SETBIT (board.b[xside][cpiece], t);  
       SETBIT (board.blockerr90, r90[t]);  
       SETBIT (board.blockerr45, r45[t]);  
@@ -322,6 +325,7 @@ void UnmakeMove (short side, int *move)
    /* Undo enpassant */
    if (*move & ENPASSANT)
    {
+      ExchCnt[side]--;
       epsq = (side == white ? g->epsq - 8 : g->epsq + 8);
       SETBIT (board.b[xside][pawn], epsq); 
       SETBIT (board.blockerr90, r90[epsq]); 
@@ -513,9 +517,24 @@ leaf * ValidateMove (char *s)
    FilterIllegalMoves (1);	
    side = board.side;
 
+   /************************************************************************
+    * The thing to do now is to clean up the move string.  This
+    * gets rid of things like 'x', '+', '=' and so forth.
+    ************************************************************************/
+   p = mvstr;
+   do
+   {
+      if (*s != 'x' && *s != '+' && *s != '=')
+         *p++ = *s; 
+   } while (*s++ != '\0');
+
+   /* Flush castles that check */
+   if (mvstr[strlen(mvstr)-1] == '+' || mvstr[strlen(mvstr)-1] == '#' ||
+       mvstr[strlen(mvstr)-1] == '=') mvstr[strlen(mvstr)-1] = '\000'; 
+
    /* Check for castling */
-   if (strcmp (s, "O-O") == 0 || strcmp (s, "o-o") == 0 || 
-       strcmp (s, "0-0") == 0)
+   if (strcmp (mvstr, "O-O") == 0 || strcmp (mvstr, "o-o") == 0 || 
+       strcmp (mvstr, "0-0") == 0)
    {
       if (side == white)
       {
@@ -528,8 +547,8 @@ leaf * ValidateMove (char *s)
       return (IsInMoveList (1, f, t, ' '));
    }
 
-   if (strcmp (s, "O-O-O") == 0 || strcmp (s, "o-o-o") == 0 ||
-       strcmp (s, "0-0-0") == 0)
+   if (strcmp (mvstr, "O-O-O") == 0 || strcmp (mvstr, "o-o-o") == 0 ||
+       strcmp (mvstr, "0-0-0") == 0)
    {
       if (side == white)
       {
@@ -542,17 +561,6 @@ leaf * ValidateMove (char *s)
       return (IsInMoveList (1, f, t, ' '));
    }
 
-   /************************************************************************
-    * The thing to do now is to clean up the move string.  This
-    * gets rid of things like 'x', '+', '=' and so forth.
-    ************************************************************************/
-   p = mvstr;
-   do
-   {
-      if (*s != 'x' && *s != '+' && *s != '=')
-         *p++ = *s; 
-   } while (*s++ != '\0');
-
    /*  Test to see if it is e2e4 type notation */
    if (ATOH (mvstr[0]) && ITO8 (mvstr[1]) && ATOH (mvstr[2]) &&
 	ITO8 (mvstr[3]))
@@ -614,7 +622,12 @@ leaf * ValidateMove (char *s)
 	    	      f = t - (side == white ? 8 : -8) + (file - fileto);
          	      piece = (strlen (mvstr) == 3 ? mvstr[2] : ' ');
 	    	      return (IsInMoveList (1, f, t, piece));
-	    default : printf ("Ambiguous move\n");
+	    default : 
+		      printf ("Ambiguous move: %s %s\n",s,mvstr);
+		      ShowBoard();
+/*
+		      getchar();
+*/
 	    	      return ((leaf *) NULL);
 	 }
       } 
@@ -662,7 +675,11 @@ leaf * ValidateMove (char *s)
       }
       if (kount > 1)
       {
-         printf ("Ambiguous move\n");
+	 printf ("Ambiguous move: %s %s\n",s,mvstr);
+	 ShowBoard();
+/*
+	 getchar();
+*/
    	 return ((leaf *) NULL);
       } 
       else if (kount == 0)
