@@ -45,12 +45,18 @@ inline void ShowThinking (leaf *p, short ply)
       return;
    if (!(flags & POST))
       return;
+   if (NodeCnt < 500000 && (flags & SOLVE)) {
+      /* printf("NodeCnt = %d\n",NodeCnt); getchar(); */
+      return;
+   }
    SANMove (p->move, ply);
    fprintf (stderr, "\r%2d.         %2d/%2d%10s    ", Idepth/DEPTH, 
       (int) (p-TreePtr[ply]+1), (int) (TreePtr[ply+1]-TreePtr[ply]), SANmv);
    fflush (stderr);
 }
 
+static const BitBoard brank67[2] = { 0x0000000000FFFF00ULL,
+                                     0x00FFFF0000000000ULL };
 static const short rank7[2] = { 6, 1 };
 static const short rank6[2] = { 5, 2 };
 static const char ntype[3][4] = {"PV", "ALL", "CUT"}; 
@@ -78,7 +84,7 @@ int SearchRoot (short depth, int alpha, int beta)
    KingThrt[white][ply] = MateScan (white);
    KingThrt[black][ply] = MateScan (black);
    InChk[ply] = SqAtakd (board.king[side], xside);
-   if (InChk[ply] && ChkCnt[ply] < Idepth/DEPTH)
+   if (InChk[ply] && ChkCnt[ply] < 3*Idepth/DEPTH)
    {
       ChkExtCnt++;
       ChkCnt[ply+1]++;
@@ -286,17 +292,27 @@ int Search (short ply, short depth, int alpha, int beta, short nodetype)
       extend = true;
       donull = false;
    }
-/*
-   else if (g1 & (CAPTURE | PROMOTION) && t0 == t1 && ply <= Idepth/DEPTH+2)
-*/
-   else if (g0 & (CAPTURE | PROMOTION) && (board.material[computer] - 
+   /* Promotion extension */
+   else if (g0 & PROMOTION)
+   {
+      PawnExtCnt++;
+      depth += DEPTH;
+      extend = true;
+   }
+   /* Recapture extension */
+   else if ((g0 & CAPTURE) && (board.material[computer] - 
 	board.material[1^computer] == RootMaterial))
    {
       RcpExtCnt++;
       depth += DEPTH;
       extend = true;
    }
+/* OLD
    else if (depth <= DEPTH && cboard[t0] == pawn && (RANK(t0) == rank7[xside]))
+   else if (depth <= DEPTH && cboard[t0] == pawn && (BitPosArray[t0] & brank67[xside]) != NULLBITBOARD)
+*/
+   /* 6th or 7th rank extension */
+   else if (depth <= DEPTH && cboard[t0] == pawn && (RANK(t0) == rank7[xside] || RANK(t0) == rank6[xside]))
    {
       PawnExtCnt++;
       depth += DEPTH;
@@ -701,6 +717,10 @@ void ShowLine (int move, int score, char c)
    /* SMC */
    if (!(flags & POST))
      return;
+   if (NodeCnt < 500000 && (flags & SOLVE)) {
+      /* printf("NodeCnt = %d\n",NodeCnt); getchar(); */
+      return;
+   }
    if (Idepth == DEPTH && c == '&')
       return;
    if ((flags & XBOARD) && c == '&')
